% Computes instantanous swimming velocities from flagellar beat pattern
% (c) BM Friedrich, 07.08.2017

% parameters resistive force theory ---------------------------------------
% from Friedrich et al.J exp. Biol. (2010)
zetapar=0.38e-3; % [pN s/um^2]
zetaper=1.81*zetapar; 
% zetaper=zetapar; % TEST: without drag anisotropy
% forces ... [pN], torques ... [aNm]

% parameters of beat pattern ----------------------------------------------

L=50; % [um] length of flagellum
wave_number=1;
k=2*pi*wave_number/L; % [1/um]

f=30; % [Hz] beat frequency
beat_frequency=2*pi*f; % [1/s] angular beat frequency
T=1/f; % [s] beat period

%% generate synthetic beat pattern
% ... time and arc-length step
ndt=301; % number of time steps
dt=T/(ndt-1); % [s], small time-step needed for accurate numerics
tlist=(0:ndt-1)*dt;

nds=101; % number of control points along flagelum
ds=L/(nds-1); % [um] spacing between arclength positions
slist=(0:nds-1)*ds; % [um] arclength positions

% xflag: matrix of e1-coordinates
% - row index=time index, corresponding to subsequent time points t=idt*dt
% - column index=arc-length position index, corresponding to equidistant control points along flagllum at arc-length position s=ids*ds
xflag=nan(ndt,nds); % dimension of coordinate matrix (ndt = total number of time points, nds = number of arc-length positions in each flagellar shape)
yflag=nan(ndt,nds);

for idt=1:ndt
    t=idt*dt; % time [s]
    % tanget angle    
    % psi=cos(k*slist).*cos(beat_frequency*t); % standing wave
    psi=cos(k*slist).*cos(beat_frequency*t)-sin(k*slist).*sin(beat_frequency*t); % traveling wave
    xflag(idt,:)=cumsum(cos(psi))*ds; % integrate x- and y-coordinates
    yflag(idt,:)=cumsum(sin(psi))*ds;
end % idt

%% plot flagellar shape ----------------------------------------------------
figure(1), clf, hold on
daspect([1 1 1])
for idt=1:10:ndt
    color=hsv2rgb([mod(idt/ndt,1) 1 1]); % rainbow colors
    plot(xflag(idt,:),yflag(idt,:),'color',color);
end
% saveas(gcf,'fig.png') % save figure as png-image

%% main loop --------------------------------------------------------------

% initial conditions
rr=[0 0]; % [um] position
alpha=0; % [rad] orientation angle
e1=[ cos(alpha) sin(alpha)]; % material frame (unit vector)
e2=[-sin(alpha) cos(alpha)];

% swimming path
rrlist=nan(ndt,2); % [um]

% loop over subsequent time points
% for idt=10
for idt=1:ndt-1
    
    x1=xflag(idt,:);
    x2=yflag(idt,:);
    
    % tangent angle
    psi=atan2(gradient(x2),gradient(x1));
    
    % tangent vector (unit length)
    t1=-cos(psi); % components with respect to (e1,e2) material frame
    t2=-sin(psi);

    % normal vector
    n1=-t2;
    n2= t1;

    % velocity profile along flagellar shape [in (e1,e2)-coordinate system]
    v1=( xflag(idt+1,:)-xflag(idt,:) )/dt; % [um/s]
    v2=( yflag(idt+1,:)-yflag(idt,:) )/dt; % [um/s]
    
    % parallel component of velocity profile
    vpar1=(v1.*t1+v2.*t2).*t1; % [um/s]
    vpar2=(v1.*t1+v2.*t2).*t2; % [um/s]
    
    % perpendicular component 
    vper1=(v1.*n1+v2.*n2).*n1; % [um/s]
    vper2=(v1.*n1+v2.*n2).*n2; % [um/s]
    
    % forces densities
    f1=zetapar*vpar1+zetaper*vper1; % [pN/um]
    f2=zetapar*vpar2+zetaper*vper2; % [pN/um]
    
    % forces
    F1=sum(f1)*ds; % [pN]
    F2=sum(f2)*ds; % [pN]

    % torque
    M3=sum(f1.*x2-f2.*x1)*ds; % [pN um]

    % right-hand side of linear equation system to be solved below
    rhs=[F1;F2;M3];
    
    % print to command window
    fprintf(1,'F1=%.2f pN, F2=%.2f pN, M3=%.2f pN um\n',F1,F2,M3)

    % resistance matrix
    
    % first column: translation in e1-direction
    v1=ones(1,nds);  % = row vector of ones
    v2=zeros(1,nds); % = row vector of zeros
        
    % parallel component of velocity profile
    vpar1=(v1.*t1+v2.*t2).*t1; % [dimension-less]
    vpar2=(v1.*t1+v2.*t2).*t2; % [dimension-less]
    
    % perpendicular component 
    vper1=(v1.*n1+v2.*n2).*n1;
    vper2=(v1.*n1+v2.*n2).*n2;
    
    % forces densities
    f1=zetapar*vpar1+zetaper*vper1; % [pN/um]
    f2=zetapar*vpar2+zetaper*vper2; % [pN/um]
    
    % forces
    F1=sum(f1)*ds; % [pN]
    F2=sum(f2)*ds;

    % torque
    M3=sum(f1.*x2-f2.*x1)*ds; % [pN um]

    Gamma(:,1)=[F1;F2;M3];
            
    % second column: translation in e2-directino    
    v1=zeros(1,nds);
    v2=ones(1,nds);
        
    % parallel component of velocity profile
    vpar1=(v1.*t1+v2.*t2).*t1; % [um/s]
    vpar2=(v1.*t1+v2.*t2).*t2; % [um/s]
    
    % perpendicular component 
    vper1=(v1.*n1+v2.*n2).*n1;
    vper2=(v1.*n1+v2.*n2).*n2;
    
    % forces densities
    f1=zetapar*vpar1+zetaper*vper1; % [pN/um]
    f2=zetapar*vpar2+zetaper*vper2; % [pN/um]
    
    % forces
    F1=sum(f1)*ds; % [pN]
    F2=sum(f2)*ds;

    % torque
    M3=sum(f1.*x2-f2.*x1)*ds;

    Gamma(:,2)=[F1;F2;M3];

    % third column: rotation
    v1=-x2;
    v2= x1;
           
    % parallel component of velocity profile
    vpar1=(v1.*t1+v2.*t2).*t1; % [um/s]
    vpar2=(v1.*t1+v2.*t2).*t2; % [um/s]
    
    % perpendicular component 
    vper1=(v1.*n1+v2.*n2).*n1;
    vper2=(v1.*n1+v2.*n2).*n2;
    
    % forces densities
    f1=zetapar*vpar1+zetaper*vper1; % [pN/um]
    f2=zetapar*vpar2+zetaper*vper2; % [pN/um]
    
    % forces
    F1=sum(f1)*ds; % [pN]
    F2=sum(f2)*ds;

    % torque
    M3=sum(f1.*x2-f2.*x1)*ds;

    Gamma(:,3)=[F1;F2;M3];
    
    % solve liner system
    sol=-inv(Gamma)*rhs;
        
    % store speeds
    v1=sol(1); % [um/s] translational velocity in e1-direction
    v2=sol(2); % [um/s] translational velocity in e2-direction
    w =sol(3); % [1/s] rotational velocity for rotations around e3=e2 x e1
    
    % update orientation
    alpha=alpha+w*dt;
    
    % update material frame
    e1=[ cos(alpha) sin(alpha)]; % material frame
    e2=[-sin(alpha) cos(alpha)];

    % update position
    rr=rr+(v1*e1+v2*e2)*dt;         
    
    % transform flagellar shape to lab frame
    xflag_lab(idt,:)=rr(1)+x1*e1(1)+x2*e2(1);
    yflag_lab(idt,:)=rr(2)+x1*e1(2)+x2*e2(2);
    
    % keep a record
    rrlist(idt,:)=rr;    
    
end % idt

%% plot trajectory

figure(2), clf, hold on
% trajectory
plot(rrlist(:,1),rrlist(:,2),'b','LineWidth',2)
axis equal
xlabel('x [um]')
ylabel('y [um]')

% flagellar shapes
for idt=1:10:ndt-1
    color=hsv2rgb([mod(idt/ndt,1) 1 1]); % rainbow colors
    plot(xflag_lab(idt,:),yflag_lab(idt,:),'color',color)
    plot(mean(xflag_lab(idt,:)),mean(yflag_lab(idt,:)),'r.','MarkerSize',20) % center of mass
end